// (C) 2023 GoodData Corporation
import React, { createContext, useContext, useEffect, useState, useMemo } from "react";
import noop from "lodash/noop.js";
import isEmpty from "lodash/isEmpty.js";
import stableStringify from "json-stable-stringify";
import { useBackendProvider } from "./useBackendProvider.js";
import { appendEmptyAttribute, convertToCatalogAttributeDataByRefs, findCatalogAttributeByRef, } from "./utils.js";
export const AttributeHierarchyDialogProviderContext = createContext({
    isEditing: false,
    isDirty: false,
    attributes: [],
    title: "",
    isLoading: true,
    setLoading: noop,
    shouldDisplayDeleteConfirmation: false,
    setDisplayDeleteConfirmation: noop,
    onClose: noop,
    onUpdateTitle: noop,
    onAddEmptyAttribute: noop,
    onCompleteAttribute: noop,
    onDeleteAttribute: noop,
    onSaveAttributeHierarchy: noop,
    onDeleteAttributeHierarchy: noop,
    getValidAttributes: () => Promise.resolve([]),
});
export const useAttributeHierarchyDialog = () => useContext(AttributeHierarchyDialogProviderContext);
export const AttributeHierarchyDialogProvider = ({ initialAttributeRef, editingAttributeHierarchy, children, onClose, onSaveOrUpdateSuccess, onDeleteSuccess, onAddAttributeClicked, onCreateHierarchyClicked, }) => {
    const initialTitle = editingAttributeHierarchy?.attributeHierarchy?.title ?? "";
    const [isLoading, setLoading] = useState(true);
    const [title, setTitle] = useState(initialTitle);
    const [initialAttributes, setInitialAttributes] = useState();
    const [attributes, setAttributes] = useState([]);
    const [shouldDisplayDeleteConfirmation, setDisplayDeleteConfirmation] = useState();
    const isEditing = !!editingAttributeHierarchy;
    const isDirty = useMemo(() => {
        return initialTitle !== title || stableStringify(attributes) !== stableStringify(initialAttributes);
    }, [initialTitle, title, attributes, initialAttributes]);
    const { catalogAttributesMap, onSaveAttributeHierarchy, onDeleteAttributeHierarchy, getValidAttributes } = useBackendProvider({
        setLoading,
        title,
        attributes,
        editingAttributeHierarchy,
        onClose,
        onSaveOrUpdateSuccess,
        onDeleteSuccess,
        onCreateHierarchyClicked,
    });
    const handleAddEmptyAttribute = (baseRowIndex) => {
        setAttributes(appendEmptyAttribute(attributes, baseRowIndex));
        if (onAddAttributeClicked) {
            onAddAttributeClicked();
        }
    };
    const handleCompeteAttribute = (selectedAttribute, rowIndex) => {
        const attribute = {
            ...selectedAttribute,
            completed: true,
        };
        const newAttributes = [...attributes];
        newAttributes[rowIndex] = attribute;
        setAttributes(newAttributes);
    };
    const handleDeleteAttribute = (rowIndex) => {
        const newAttributes = [...attributes];
        newAttributes.splice(rowIndex, 1);
        setAttributes(newAttributes);
    };
    useEffect(() => {
        if (editingAttributeHierarchy && !isEmpty(catalogAttributesMap)) {
            const refs = editingAttributeHierarchy.attributeHierarchy.attributes;
            const initialAttributes = convertToCatalogAttributeDataByRefs(catalogAttributesMap, refs).map((attribute) => ({
                ...attribute,
                completed: true,
            }));
            setAttributes(initialAttributes);
            setInitialAttributes(initialAttributes);
            return;
        }
        if (initialAttributeRef && !isEmpty(catalogAttributesMap)) {
            const initialAttribute = findCatalogAttributeByRef(catalogAttributesMap, initialAttributeRef);
            if (initialAttribute) {
                const initialAttributes = [
                    {
                        ...initialAttribute,
                        completed: true,
                    },
                ];
                setAttributes(initialAttributes);
                setInitialAttributes(initialAttributes);
            }
        }
    }, [initialAttributeRef, editingAttributeHierarchy, catalogAttributesMap]);
    return (React.createElement(AttributeHierarchyDialogProviderContext.Provider, { value: {
            isEditing,
            isDirty,
            attributes,
            title,
            isLoading,
            setLoading,
            shouldDisplayDeleteConfirmation,
            setDisplayDeleteConfirmation,
            onClose,
            onUpdateTitle: setTitle,
            onAddEmptyAttribute: handleAddEmptyAttribute,
            onCompleteAttribute: handleCompeteAttribute,
            getValidAttributes,
            onDeleteAttribute: handleDeleteAttribute,
            onSaveAttributeHierarchy,
            onDeleteAttributeHierarchy,
        } }, children));
};
//# sourceMappingURL=AttributeHierarchyDialogProvider.js.map