// (C) 2019-2024 GoodData Corporation
import React from "react";
import { FormattedMessage, injectIntl } from "react-intl";
import noop from "lodash/noop.js";
import set from "lodash/set.js";
import cloneDeep from "lodash/cloneDeep.js";
import cx from "classnames";
import DisabledBubbleMessage from "../DisabledBubbleMessage.js";
import { getTranslation } from "../../utils/translations.js";
const MAX_NUMBER_LENGTH = 15;
export class InputControl extends React.Component {
    static defaultProps = {
        value: "",
        type: "text",
        disabled: false,
        pushData: noop,
        max: Number.MAX_SAFE_INTEGER,
        min: Number.MIN_SAFE_INTEGER,
        step: 1,
        showDisabledMessage: false,
        hasWarning: false,
        validateAndPushDataCallback: noop,
    };
    inputRef;
    constructor(props) {
        super(props);
        this.state = {
            value: props.value,
            lastSentValue: props.value,
        };
        this.onValueChanged = this.onValueChanged.bind(this);
        this.onKeyPress = this.onKeyPress.bind(this);
        this.onBlur = this.onBlur.bind(this);
        this.triggerBlur = this.triggerBlur.bind(this);
    }
    UNSAFE_componentWillReceiveProps(newProps) {
        if (newProps.value !== this.state.value) {
            this.setState({
                value: newProps.value,
                lastSentValue: newProps.value,
            });
        }
    }
    render() {
        const { disabled, labelText, placeholder, showDisabledMessage, disabledMessageId, disabledMessageAlignPoints, intl, type, maxLength, description, descriptionValues, } = this.props;
        return (React.createElement(DisabledBubbleMessage, { showDisabledMessage: showDisabledMessage, messageId: disabledMessageId, alignPoints: disabledMessageAlignPoints },
            React.createElement(React.Fragment, null,
                React.createElement("label", { className: "adi-bucket-inputfield s-adi-bucket-inputfield gd-input gd-input-small" },
                    React.createElement("span", { className: "input-label-text" }, getTranslation(labelText, intl)),
                    React.createElement("input", { ref: (input) => (this.inputRef = input), className: this.getInputClassNames(), value: this.state.value, placeholder: getTranslation(placeholder, intl), disabled: disabled, onKeyPress: this.onKeyPress, onBlur: this.onBlur, onChange: this.onValueChanged, maxLength: type === "number" ? MAX_NUMBER_LENGTH : maxLength })),
                description ? (React.createElement("div", { className: "adi-bucket-inputfield-description" },
                    React.createElement(FormattedMessage, { id: description, values: descriptionValues }))) : null)));
    }
    getInputClassNames() {
        const { type, hasWarning } = this.props;
        return cx("gd-input-field", "gd-input-field-small", {
            "has-warning": hasWarning,
            number: type === "number",
        });
    }
    isValid(type, value) {
        const { validateFn } = this.props;
        if (validateFn) {
            return validateFn(value);
        }
        if (type === "number") {
            // allow only numbers, `-` and string doesn't starts with `.`
            return !value.startsWith(".") && (!isNaN(Number(value)) || value === "-");
        }
        return true;
    }
    onValueChanged(event) {
        const { type } = this.props;
        const { value } = event.target;
        if (this.isValid(type, value)) {
            this.setState({ value });
        }
    }
    triggerBlur() {
        if (this.inputRef) {
            this.inputRef.blur();
        }
    }
    modifyDataForSending(value) {
        const { type, transformFn } = this.props;
        if (transformFn) {
            return transformFn(value);
        }
        if (type === "number") {
            return value.replace(/\.+$/, "");
        }
        return value;
    }
    emitData() {
        const { valuePath, properties, pushData } = this.props;
        const { value } = this.state;
        const modifiedData = this.modifyDataForSending(value);
        const clonedProperties = cloneDeep(properties);
        set(clonedProperties, `controls.${valuePath}`, modifiedData);
        this.setState({ value: modifiedData });
        pushData({ properties: clonedProperties });
        return modifiedData;
    }
    onBlur() {
        const { value, lastSentValue } = this.state;
        if (lastSentValue !== value) {
            const validatedData = this.emitData();
            this.setState({ lastSentValue: validatedData });
        }
    }
    onKeyPress(event) {
        if (event.key === "Enter") {
            const { value, lastSentValue } = this.state;
            if (lastSentValue !== value) {
                const validatedData = this.emitData();
                this.setState({ lastSentValue: validatedData }, this.triggerBlur);
            }
            else {
                this.triggerBlur();
            }
        }
    }
}
export default injectIntl(InputControl);
//# sourceMappingURL=InputControl.js.map