// (C) 2024-2025 GoodData Corporation
import React, { useRef, useEffect } from "react";
import { EditorView, keymap } from "@codemirror/view";
import { EditorState } from "@codemirror/state";
import { yaml } from "@codemirror/lang-yaml";
import { syntaxHighlighting, defaultHighlightStyle } from "@codemirror/language";
import { defaultKeymap, history, historyKeymap, indentWithTab } from "@codemirror/commands";
import { linter, lintGutter } from "@codemirror/lint";
import * as jsYaml from "js-yaml";
// YAML validator
function yamlValidator(view) {
    const errors = [];
    try {
        jsYaml.load(view.state.doc.toString());
    }
    catch (e) {
        errors.push({
            from: e.mark.position - 1,
            to: e.mark.position - 1,
            message: e.message,
            severity: "error",
        });
    }
    return errors;
}
export const CodeMirrorEditor = ({ value, onChange }) => {
    const editorRef = useRef(null);
    const viewRef = useRef();
    useEffect(() => {
        if (!editorRef.current) {
            return undefined;
        }
        const changeHandler = EditorView.updateListener.of((update) => {
            if (update.docChanged) {
                const newValue = update.state.doc.toString();
                onChange(newValue);
            }
        });
        const view = new EditorView({
            state: EditorState.create({
                doc: viewRef.current?.state.doc.toString() ?? "",
                extensions: [
                    history(),
                    syntaxHighlighting(defaultHighlightStyle, { fallback: true }),
                    keymap.of([indentWithTab, ...defaultKeymap, ...historyKeymap]),
                    yaml(),
                    EditorView.lineWrapping,
                    linter(yamlValidator, { markerFilter: () => [] }),
                    lintGutter(),
                    changeHandler,
                ],
            }),
            parent: editorRef.current,
        });
        viewRef.current = view;
        return () => view.destroy();
    }, [onChange]);
    useEffect(() => {
        const view = viewRef.current;
        if (!view)
            return;
        const currentValue = view.state.doc.toString();
        if (currentValue !== value) {
            view.dispatch({
                changes: { from: 0, to: currentValue.length, insert: value || "" },
            });
        }
    }, [value]);
    return React.createElement("div", { className: "gd-advanced-customization-dialog__theme", ref: editorRef });
};
//# sourceMappingURL=CodeMirrorEditor.js.map