// (C) 2022-2025 GoodData Corporation
import React, { useCallback, useEffect, useMemo, useState } from "react";
import { IntlWrapper } from "@gooddata/sdk-ui";
import { EmbedInsightDialogBase, Overlay, getDefaultEmbedTypeOptions, getHeightWithUnitsForEmbedCode, } from "@gooddata/sdk-ui-kit";
import { FullVisualizationCatalog } from "../../VisualizationCatalog.js";
import { insightViewEmbeddedCodeGenerator } from "../../../utils/embeddingInsightViewCodeGenerator/insightViewCodeGenerator.js";
import { getWebComponentsCodeGenerator } from "../../../utils/embeddingCodeGenerator/getWebComponentsCodeGenerator.js";
const INSIGHT_VIEW_PROPERTIES_LINK = "https://sdk.gooddata.com/gooddata-ui/docs/visualization_component.html#properties";
/**
 * @internal
 */
export function EmbedInsightDialog(props) {
    const { locale, openSaveInsightDialog, onClose, onCopyCode, showWebComponentsTab } = props;
    const { code, documentationLink, integrationDocLink, currentTab, codeOption, onTabChange, onOptionsChange, } = useEmbedInsightDialog(props);
    return (React.createElement(IntlWrapper, { locale: locale },
        React.createElement(ModalOverlay, null,
            React.createElement(EmbedInsightDialogBase, { code: code, propertiesLink: documentationLink, showWebComponentsTab: showWebComponentsTab, integrationDocLink: integrationDocLink, openSaveInsightDialog: openSaveInsightDialog, onClose: onClose, onCopyCode: onCopyCode, embedTab: currentTab, onTabChange: onTabChange, embedTypeOptions: codeOption, onOptionsChange: onOptionsChange }))));
}
const useEmbedInsightDialog = (props) => {
    const { insight, backend, settings, colorPalette, theme, executionConfig, reactIntegrationDocLink, webComponentIntegrationDocLink, workspaceId, } = props;
    const [currentTab, setCurrentTab] = useState("react");
    const [codeOption, setCodeOption] = useState(getDefaultEmbedTypeOptions(currentTab));
    useEffect(() => {
        setCodeOption(getDefaultEmbedTypeOptions(currentTab));
    }, [currentTab]);
    const code = useMemo(() => {
        const isReactDefinitionCode = codeOption.type === "react" && codeOption.componentType === "definition";
        if (!insight.insight.identifier && !isReactDefinitionCode) {
            return null;
        }
        const inputBase = {
            insight,
            settings,
            backend,
            colorPalette,
            theme,
            executionConfig,
            workspaceId,
        };
        return codeOption.type === "react"
            ? generateCodeByReact({
                ...inputBase,
                codeOption: codeOption,
            })
            : generateCodeByWebComponents({
                ...inputBase,
                codeOption,
            });
    }, [codeOption, insight, settings, backend, colorPalette, theme, executionConfig, workspaceId]);
    const documentationLink = useMemo(() => {
        if (codeOption.type === "react") {
            return getLinkToPropertiesDocumentation(codeOption.componentType, insight);
        }
        return getLinkToPropertiesDocumentation("definition", insight);
    }, [insight, codeOption]);
    const integrationDocLink = useMemo(() => {
        return currentTab === "react" ? reactIntegrationDocLink : webComponentIntegrationDocLink;
    }, [currentTab, reactIntegrationDocLink, webComponentIntegrationDocLink]);
    const onTabChange = useCallback((selectedTab) => {
        setCurrentTab(selectedTab);
    }, []);
    const onOptionsChange = useCallback((opt) => setCodeOption(opt), []);
    return {
        code,
        documentationLink,
        integrationDocLink,
        currentTab,
        codeOption,
        onTabChange,
        onOptionsChange,
    };
};
const getLinkToPropertiesDocumentation = (codeType, insight) => {
    if (codeType === "definition") {
        const meta = FullVisualizationCatalog.forInsight(insight).getMeta();
        if (meta?.documentationUrl) {
            return `${meta?.documentationUrl}#properties`;
        }
    }
    return INSIGHT_VIEW_PROPERTIES_LINK;
};
const generateCodeByReact = (input) => {
    const { backend, codeOption, colorPalette, theme, executionConfig, insight, settings } = input;
    const height = getHeightWithUnitsForEmbedCode(codeOption);
    const generateCodeConfig = {
        context: {
            settings,
            backend,
            colorPalette,
            theme,
            executionConfig,
        },
        language: codeOption.codeType,
        height,
        omitChartProps: codeOption.displayConfiguration ? [] : ["config"],
    };
    if (codeOption.componentType === "definition") {
        const descriptor = FullVisualizationCatalog.forInsight(insight);
        return descriptor.getEmbeddingCode?.(insight, generateCodeConfig);
    }
    return insightViewEmbeddedCodeGenerator(insight, generateCodeConfig);
};
const generateCodeByWebComponents = (input) => {
    const { codeOption, insight, workspaceId } = input;
    const height = getHeightWithUnitsForEmbedCode(codeOption);
    return getWebComponentsCodeGenerator(workspaceId, insight, { ...codeOption, height });
};
const BUBBLE_ALIGN_POINTS = [{ align: "cc cc" }];
const ModalOverlay = (props) => {
    const { children } = props;
    return (React.createElement(Overlay, { alignPoints: BUBBLE_ALIGN_POINTS, isModal: true, positionType: "fixed" }, children));
};
//# sourceMappingURL=EmbedInsightDialog.js.map