// (C) 2023-2024 GoodData Corporation
import React, { useMemo, useState } from "react";
import { useIntl } from "react-intl";
import { LoadingComponent } from "@gooddata/sdk-ui";
import { Tabs, Overlay, Message } from "@gooddata/sdk-ui-kit";
import { userDialogTabsMessages, messages } from "./locales.js";
import { WorkspaceList } from "./Workspace/WorkspaceList.js";
import { AddWorkspace } from "./Workspace/AddWorkspace.js";
import { UserDetailsView } from "./Details/UserDetailsView.js";
import { UserGroupsList } from "./UserGroups/UserGroupsList.js";
import { EditUserDetails } from "./Details/EditUserDetails.js";
import { AddUserGroup } from "./UserGroups/AddUserGroup.js";
import { useUserDialogTabs, useUserGroups, useUser, useUserDialogMode, useDeleteUser, useDeleteDialog, useOrganizationDetails, } from "./dialogHooks.js";
import { ViewDialog } from "./ViewDialog.js";
import { DeleteConfirmDialog } from "./ConfirmDialogs/DeleteConfirmDialog.js";
import { OrganizationIdProvider } from "./OrganizationIdContext.js";
import { extractUserName } from "./utils.js";
import { withTelemetry } from "./TelemetryContext.js";
import { DataSourceList } from "./DataSources/DataSourceList.js";
import { AddDataSource } from "./DataSources/AddDataSource.js";
import { usePermissions } from "./hooks/usePermissions.js";
import { ErrorDialog } from "./ErrorDialog.js";
import { isUnexpectedResponseError } from "@gooddata/sdk-backend-spi";
const alignPoints = [{ align: "cc cc" }];
const UserEditDialogComponent = ({ userId, organizationId, isAdmin, onSuccess, onClose, initialView = "VIEW", changeUserMembership = false, renderDataSourceIcon, areFilterViewsEnabled = false, selectedTab = undefined, }) => {
    const intl = useIntl();
    const { dialogMode, setDialogMode } = useUserDialogMode(initialView);
    const { user, isCurrentlyAdmin, onUserDetailsChanged, setIsAdmin, error, isLoading: userIsLoading, } = useUser(userId, organizationId, isAdmin, onSuccess);
    const { isBootstrapUser, bootstrapUserGroupId } = useOrganizationDetails(organizationId);
    const { grantedWorkspaces, onWorkspacesChanged, removeGrantedWorkspace, updateGrantedWorkspace, grantedDataSources, onDataSourcesChanged, removeGrantedDataSource, updateGrantedDataSource, } = usePermissions(userId, "user", organizationId, onSuccess);
    const { grantedUserGroups, onUserGroupsChanged, removeGrantedUserGroup, removeAdminGroup } = useUserGroups(userId, organizationId, bootstrapUserGroupId, onSuccess, setIsAdmin);
    const { tabs, selectedTabId, setSelectedTabId } = useUserDialogTabs(grantedWorkspaces, grantedUserGroups, grantedDataSources, isCurrentlyAdmin, selectedTab);
    const { isConfirmDeleteOpened, onOpenDeleteDialog, onCloseDeleteDialog, dialogOverlayClassNames, dialogWrapperClassNames, } = useDeleteDialog();
    const { deleteUser, isDeleteUserProcessing } = useDeleteUser(userId, organizationId, onSuccess, onClose);
    const { editButtonText, editButtonMode, editButtonIconClassName } = useMemo(() => {
        if (selectedTabId.id === userDialogTabsMessages.workspaces.id) {
            return {
                editButtonMode: "WORKSPACE",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addWorkspaceButton),
            };
        }
        if (selectedTabId.id === userDialogTabsMessages.userGroups.id) {
            return {
                editButtonMode: "USER_GROUPS",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addUserGroupButton),
            };
        }
        if (selectedTabId.id === userDialogTabsMessages.dataSources.id) {
            return {
                editButtonMode: "DATA_SOURCES",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addDataSourcePermissionButton),
            };
        }
        return {
            editButtonMode: "DETAIL",
            editButtonIconClassName: "gd-icon-pencil gd-user-management-dialog-edit-mode-icon",
            editButtonText: intl.formatMessage(messages.editUserButton),
        };
    }, [intl, selectedTabId]);
    const isLoading = error
        ? false
        : userIsLoading || !grantedWorkspaces || !grantedUserGroups || !grantedDataSources;
    const isOpenedInEditMode = initialView !== "VIEW";
    const [workspaceToEdit, setWorkspaceToEdit] = useState(undefined);
    const handleWorkspaceEdit = (workspace) => {
        setWorkspaceToEdit(workspace);
        setDialogMode("WORKSPACE");
    };
    const handleWorkspaceChanged = (workspaces) => {
        setWorkspaceToEdit(undefined);
        onWorkspacesChanged(workspaces);
    };
    const handleWorkspaceCancel = () => {
        setWorkspaceToEdit(undefined);
        setDialogMode("VIEW");
    };
    return (React.createElement(OrganizationIdProvider, { organizationId: organizationId },
        isConfirmDeleteOpened ? (React.createElement(DeleteConfirmDialog, { titleText: intl.formatMessage(messages.deleteUserConfirmTitle), bodyText: intl.formatMessage(messages.deleteUserConfirmBody), onConfirm: deleteUser, isProcessing: isDeleteUserProcessing, onCancel: onCloseDeleteDialog })) : null,
        React.createElement(Overlay, { alignPoints: alignPoints, isModal: true, positionType: "fixed", className: dialogOverlayClassNames, resizeObserverThreshold: 0.2 },
            React.createElement("div", { className: dialogWrapperClassNames }, isLoading ? (React.createElement(React.Fragment, null,
                React.createElement(LoadingComponent, { className: "gd-user-management-dialog-loading" }),
                React.createElement(ErrorDialog, { dialogTitle: "", onClose: onClose }))) : error ? (React.createElement(ErrorDialog, { onClose: onClose, dialogTitle: intl.formatMessage(messages.userLoadingErrorTitle) },
                React.createElement(Message, { type: "error", className: "gd-user-management-dialog-error" }, intl.formatMessage(isUnexpectedResponseError(error) && error.httpStatus === 404
                    ? messages.userLoadingErrorNotExist
                    : messages.userLoadingErrorUnknown)))) : (React.createElement(React.Fragment, null,
                dialogMode === "VIEW" && (React.createElement(ViewDialog, { dialogTitle: extractUserName(user), isAdmin: isCurrentlyAdmin, isDeleteLinkEnabled: !isBootstrapUser(user), deleteLinkDisabledTooltipTextId: messages.deleteUserDisabledTooltip.id, deleteLinkText: intl.formatMessage(messages.deleteUserLink), onOpenDeleteDialog: onOpenDeleteDialog, onClose: onClose, onEdit: () => setDialogMode(editButtonMode), editButtonText: editButtonText, editButtonIconClassName: editButtonIconClassName },
                    isCurrentlyAdmin ? (React.createElement("div", { className: "gd-message progress gd-user-management-admin-alert s-admin-message" },
                        React.createElement("div", { className: "gd-message-text" }, intl.formatMessage(messages.adminAlert)))) : null,
                    React.createElement(Tabs, { selectedTabId: selectedTabId.id, onTabSelect: setSelectedTabId, tabs: tabs, className: "gd-user-management-dialog-tabs s-user-management-tabs" }),
                    selectedTabId.id === userDialogTabsMessages.workspaces.id && (React.createElement(WorkspaceList, { workspaces: grantedWorkspaces, subjectType: "user", mode: "VIEW", onDelete: removeGrantedWorkspace, onChange: updateGrantedWorkspace, areFilterViewsEnabled: areFilterViewsEnabled, onClick: handleWorkspaceEdit })),
                    selectedTabId.id === userDialogTabsMessages.userGroups.id && (React.createElement(UserGroupsList, { userGroups: grantedUserGroups, mode: "VIEW", onDelete: removeGrantedUserGroup, isBootstrapUser: isBootstrapUser(user), bootstrapUserGroupId: bootstrapUserGroupId })),
                    selectedTabId.id === userDialogTabsMessages.details.id && (React.createElement(UserDetailsView, { user: user, isAdmin: isCurrentlyAdmin, isBootstrapUser: isBootstrapUser(user), mode: "VIEW" })),
                    selectedTabId.id === userDialogTabsMessages.dataSources.id && (React.createElement(DataSourceList, { dataSources: grantedDataSources, subjectType: "user", mode: "VIEW", onDelete: removeGrantedDataSource, onChange: updateGrantedDataSource, renderDataSourceIcon: renderDataSourceIcon })))),
                dialogMode === "WORKSPACE" && (React.createElement(AddWorkspace, { ids: [userId], subjectType: "user", grantedWorkspaces: grantedWorkspaces, enableBackButton: !isOpenedInEditMode, onSubmit: handleWorkspaceChanged, onCancel: isOpenedInEditMode ? onClose : handleWorkspaceCancel, onClose: onClose, areFilterViewsEnabled: areFilterViewsEnabled, editWorkspace: workspaceToEdit })),
                dialogMode === "USER_GROUPS" && (React.createElement(AddUserGroup, { userIds: [userId], grantedUserGroups: grantedUserGroups, enableBackButton: !isOpenedInEditMode, onSubmit: onUserGroupsChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose })),
                dialogMode === "DATA_SOURCES" && (React.createElement(AddDataSource, { ids: [userId], subjectType: "user", grantedDataSources: grantedDataSources, enableBackButton: !isOpenedInEditMode, onSubmit: onDataSourcesChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose, renderDataSourceIcon: renderDataSourceIcon })),
                dialogMode === "DETAIL" && (React.createElement(EditUserDetails, { user: user, isAdmin: isCurrentlyAdmin, isBootstrapUser: isBootstrapUser(user), enableBackButton: !isOpenedInEditMode, changeUserMembership: changeUserMembership, onSubmit: onUserDetailsChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose, removeAdminGroup: removeAdminGroup }))))))));
};
/**
 * @internal
 */
export const UserEditDialog = withTelemetry(UserEditDialogComponent);
//# sourceMappingURL=UserEditDialog.js.map