// (C) 2023-2024 GoodData Corporation
import React, { useMemo, useState } from "react";
import { useIntl } from "react-intl";
import { LoadingComponent } from "@gooddata/sdk-ui";
import { Tabs, Overlay, Message } from "@gooddata/sdk-ui-kit";
import { userGroupDialogTabsMessages, messages } from "./locales.js";
import { WorkspaceList } from "./Workspace/WorkspaceList.js";
import { AddWorkspace } from "./Workspace/AddWorkspace.js";
import { useDeleteDialog, useUserGroup, useUsers, useDeleteUserGroup, useUserGroupDialogTabs, useUserGroupDialogMode, useOrganizationDetails, } from "./dialogHooks.js";
import { ViewDialog } from "./ViewDialog.js";
import { DeleteConfirmDialog } from "./ConfirmDialogs/DeleteConfirmDialog.js";
import { OrganizationIdProvider } from "./OrganizationIdContext.js";
import { AddUser } from "./Users/AddUser.js";
import { EditUserGroupDetails } from "./Details/EditUserGroupDetails.js";
import { UserGroupDetailsView } from "./Details/UserGroupDetailsView.js";
import { UsersList } from "./Users/UsersList.js";
import { extractUserGroupName } from "./utils.js";
import { withTelemetry } from "./TelemetryContext.js";
import { DataSourceList } from "./DataSources/DataSourceList.js";
import { AddDataSource } from "./DataSources/AddDataSource.js";
import { usePermissions } from "./hooks/usePermissions.js";
import { ErrorDialog } from "./ErrorDialog.js";
import { isUnexpectedResponseError } from "@gooddata/sdk-backend-spi";
const alignPoints = [{ align: "cc cc" }];
const UserGroupEditDialogComponent = ({ userGroupId, organizationId, isAdmin, onSuccess, onClose, initialView = "VIEW", renderDataSourceIcon, areFilterViewsEnabled = false, }) => {
    const intl = useIntl();
    const { dialogMode, setDialogMode } = useUserGroupDialogMode(initialView);
    const { userGroup, onUserGroupDetailsChanged, error, isLoading: userGroupIsLoading, } = useUserGroup(userGroupId, organizationId, onSuccess);
    const { grantedWorkspaces, onWorkspacesChanged, removeGrantedWorkspace, updateGrantedWorkspace, grantedDataSources, onDataSourcesChanged, removeGrantedDataSource, updateGrantedDataSource, } = usePermissions(userGroupId, "userGroup", organizationId, onSuccess);
    const { grantedUsers, onUsersChanged, removeGrantedUsers } = useUsers(userGroupId, organizationId, onSuccess);
    const { tabs, selectedTabId, setSelectedTabId } = useUserGroupDialogTabs(grantedWorkspaces, grantedUsers, grantedDataSources, isAdmin);
    const { isConfirmDeleteOpened, onOpenDeleteDialog, onCloseDeleteDialog, dialogOverlayClassNames, dialogWrapperClassNames, } = useDeleteDialog();
    const { deleteUserGroup, isDeleteUserGroupProcessing } = useDeleteUserGroup(userGroupId, organizationId, onSuccess, onClose);
    const { isBootstrapUserGroup, bootstrapUserId } = useOrganizationDetails(organizationId);
    const { editButtonText, editButtonMode, editButtonIconClassName } = useMemo(() => {
        if (selectedTabId.id === userGroupDialogTabsMessages.workspaces.id) {
            return {
                editButtonMode: "WORKSPACE",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addWorkspaceButton),
            };
        }
        if (selectedTabId.id === userGroupDialogTabsMessages.dataSources.id) {
            return {
                editButtonMode: "DATA_SOURCES",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addDataSourcePermissionButton),
            };
        }
        if (selectedTabId.id === userGroupDialogTabsMessages.users.id) {
            return {
                editButtonMode: "USERS",
                editButtonIconClassName: "gd-icon-add",
                editButtonText: intl.formatMessage(messages.addUserButton),
            };
        }
        return {
            editButtonMode: "DETAIL",
            editButtonIconClassName: "gd-icon-pencil gd-user-management-dialog-edit-mode-icon",
            editButtonText: intl.formatMessage(messages.editUserGroupButton),
        };
    }, [intl, selectedTabId]);
    const isOpenedInEditMode = initialView !== "VIEW";
    const [workspaceToEdit, setWorkspaceToEdit] = useState(undefined);
    const handleWorkspaceEdit = (workspace) => {
        setWorkspaceToEdit(workspace);
        setDialogMode("WORKSPACE");
    };
    const handleWorkspaceChanged = (workspaces) => {
        setWorkspaceToEdit(undefined);
        onWorkspacesChanged(workspaces);
    };
    const handleWorkspaceCancel = () => {
        setWorkspaceToEdit(undefined);
        setDialogMode("VIEW");
    };
    const isLoading = error
        ? false
        : userGroupIsLoading || !grantedWorkspaces || !grantedUsers || !grantedDataSources;
    return (React.createElement(OrganizationIdProvider, { organizationId: organizationId },
        isConfirmDeleteOpened ? (React.createElement(DeleteConfirmDialog, { titleText: extractUserGroupName(userGroup), bodyText: intl.formatMessage(messages.deleteUserGroupConfirmBody), isProcessing: isDeleteUserGroupProcessing, onConfirm: deleteUserGroup, onCancel: onCloseDeleteDialog })) : null,
        React.createElement(Overlay, { alignPoints: alignPoints, isModal: true, positionType: "fixed", className: dialogOverlayClassNames, resizeObserverThreshold: 0.2 },
            React.createElement("div", { className: dialogWrapperClassNames }, isLoading ? (React.createElement(React.Fragment, null,
                React.createElement(LoadingComponent, { className: "gd-user-management-dialog-loading" }),
                React.createElement(ErrorDialog, { dialogTitle: "", onClose: onClose }))) : error ? (React.createElement(ErrorDialog, { onClose: onClose, dialogTitle: intl.formatMessage(messages.userGroupLoadingErrorTitle) },
                React.createElement(Message, { type: "error", className: "gd-user-management-dialog-error" }, intl.formatMessage(isUnexpectedResponseError(error) && error.httpStatus === 404
                    ? messages.userGroupLoadingErrorNotExist
                    : messages.userGroupLoadingErrorUnknown)))) : (React.createElement(React.Fragment, null,
                dialogMode === "VIEW" && (React.createElement(ViewDialog, { dialogTitle: extractUserGroupName(userGroup), isAdmin: isAdmin, isDeleteLinkEnabled: !isBootstrapUserGroup(userGroup) && grantedUsers?.length === 0, deleteLinkDisabledTooltipTextId: isBootstrapUserGroup(userGroup)
                        ? messages.deleteAdminUserGroupTooltip.id
                        : messages.deleteNonEmptyUserGroupTooltip.id, deleteLinkText: intl.formatMessage(messages.deleteUserGroupLink), onOpenDeleteDialog: onOpenDeleteDialog, onClose: onClose, onEdit: () => setDialogMode(editButtonMode), editButtonText: editButtonText, editButtonIconClassName: editButtonIconClassName },
                    isAdmin ? (React.createElement("div", { className: "gd-message progress gd-user-management-admin-alert s-admin-message" },
                        React.createElement("div", { className: "gd-message-text" }, intl.formatMessage(messages.adminGroupAlert)))) : null,
                    React.createElement(Tabs, { selectedTabId: selectedTabId.id, onTabSelect: setSelectedTabId, tabs: tabs, className: "gd-user-management-dialog-tabs s-user-management-tabs" }),
                    selectedTabId.id === userGroupDialogTabsMessages.workspaces.id && (React.createElement(WorkspaceList, { workspaces: grantedWorkspaces, subjectType: "userGroup", mode: "VIEW", onDelete: removeGrantedWorkspace, onChange: updateGrantedWorkspace, areFilterViewsEnabled: areFilterViewsEnabled, onClick: handleWorkspaceEdit })),
                    selectedTabId.id === userGroupDialogTabsMessages.dataSources.id && (React.createElement(DataSourceList, { dataSources: grantedDataSources, subjectType: "userGroup", mode: "VIEW", onDelete: removeGrantedDataSource, onChange: updateGrantedDataSource, renderDataSourceIcon: renderDataSourceIcon })),
                    selectedTabId.id === userGroupDialogTabsMessages.users.id && (React.createElement(UsersList, { users: grantedUsers, mode: "VIEW", onDelete: removeGrantedUsers, isBootstrapUserGroup: isBootstrapUserGroup(userGroup), bootstrapUserId: bootstrapUserId })),
                    selectedTabId.id === userGroupDialogTabsMessages.details.id && (React.createElement(UserGroupDetailsView, { userGroup: userGroup, mode: "VIEW" })))),
                dialogMode === "WORKSPACE" && (React.createElement(AddWorkspace, { ids: [userGroupId], subjectType: "userGroup", grantedWorkspaces: grantedWorkspaces, enableBackButton: !isOpenedInEditMode, onSubmit: handleWorkspaceChanged, onCancel: isOpenedInEditMode ? onClose : handleWorkspaceCancel, onClose: onClose, areFilterViewsEnabled: areFilterViewsEnabled, editWorkspace: workspaceToEdit })),
                dialogMode === "USERS" && (React.createElement(AddUser, { userGroupIds: [userGroupId], grantedUsers: grantedUsers, enableBackButton: !isOpenedInEditMode, onSubmit: onUsersChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose })),
                dialogMode === "DATA_SOURCES" && (React.createElement(AddDataSource, { ids: [userGroupId], subjectType: "userGroup", grantedDataSources: grantedDataSources, enableBackButton: !isOpenedInEditMode, onSubmit: onDataSourcesChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose, renderDataSourceIcon: renderDataSourceIcon })),
                dialogMode === "DETAIL" && (React.createElement(EditUserGroupDetails, { userGroup: userGroup, enableBackButton: !isOpenedInEditMode, onSubmit: onUserGroupDetailsChanged, onCancel: isOpenedInEditMode ? onClose : () => setDialogMode("VIEW"), onClose: onClose }))))))));
};
/**
 * @internal
 */
export const UserGroupEditDialog = withTelemetry(UserGroupEditDialogComponent);
//# sourceMappingURL=UserGroupEditDialog.js.map