// (C) 2024-2025 GoodData Corporation
import React, { useCallback, useMemo } from "react";
import { FormattedMessage, useIntl } from "react-intl";
import { DialogListHeader, Message } from "@gooddata/sdk-ui-kit";
import { AdditionalAccessPermissionItem, WorkspaceAccessPermissionItem } from "./GranularPermissionsItems.js";
import { getWorkspaceAccessPermissionDescription, workspaceGranularPermissionMessages } from "./locales.js";
import { getGranularPermissions, getWorkspacePermission, isExportPermissionIndefinite, isPermissionDisabled, workspacePermissions, sanitizeExportPermissions, removeRedundantPermissions, getImplicitGranularPermissions, } from "./granularPermissionUtils.js";
import { QuestionMarkIcon } from "./QuestionMarkIcon.js";
const granularPermissions = [
    { id: "CREATE_AUTOMATION", enabled: true },
    { id: "USE_AI_ASSISTANT", enabled: true },
    { id: "EXPORT", enabled: true },
    { id: "EXPORT_PDF", enabled: true, group: true },
    { id: "EXPORT_TABULAR", enabled: true, group: true },
    { id: "CREATE_FILTER_VIEW", enabled: true },
];
export const GranularPermissions = ({ workspace, areFilterViewsEnabled, onChange, showRedundancyWarningMessage, }) => {
    const intl = useIntl();
    const { permissions: selectedPermissions = [], isHierarchical = false } = workspace ?? {};
    const selectedWorkspacePermission = getWorkspacePermission(selectedPermissions);
    const selectedGranularPermissions = getGranularPermissions(selectedPermissions);
    const granularItems = useMemo(() => granularPermissions.filter((item) => areFilterViewsEnabled || item.id !== "CREATE_FILTER_VIEW"), [areFilterViewsEnabled]);
    const handleChange = useCallback((permissions, isHierarchical) => {
        const sanitizedPermissions = removeRedundantPermissions(permissions);
        onChange({ ...workspace, permissions: sanitizedPermissions, isHierarchical });
    }, [onChange, workspace]);
    const handleHierarchicalChange = useCallback((isHierarchical) => {
        handleChange([selectedWorkspacePermission, ...selectedGranularPermissions], isHierarchical);
    }, [handleChange, selectedGranularPermissions, selectedWorkspacePermission]);
    const handleWorkspacePermissionChange = useCallback((workspacePermission) => {
        const updatedGranularPermissions = getImplicitGranularPermissions(workspacePermission);
        handleChange([workspacePermission, ...updatedGranularPermissions], isHierarchical);
    }, [handleChange, isHierarchical]);
    const handleGranularPermissionChange = useCallback((granularPermission) => {
        const isPermissionSelected = selectedGranularPermissions.includes(granularPermission);
        const updatedGranularPermissions = isPermissionSelected
            ? selectedGranularPermissions.filter((p) => p !== granularPermission)
            : [...selectedGranularPermissions, granularPermission];
        const sanitizedGranularPermissions = sanitizeExportPermissions(granularPermission, updatedGranularPermissions, !isPermissionSelected);
        handleChange([selectedWorkspacePermission, ...sanitizedGranularPermissions], isHierarchical);
    }, [handleChange, isHierarchical, selectedGranularPermissions, selectedWorkspacePermission]);
    if (!workspace) {
        return (React.createElement("div", { className: "gd-granular-permissions-empty" },
            React.createElement(FormattedMessage, { id: "userManagement.workspace.granularPermission.noWorkspace" })));
    }
    return (React.createElement("div", { className: "gd-granular-permissions" },
        React.createElement("div", { className: "gd-granular-permissions__hierarchy" },
            React.createElement("label", { className: "input-checkbox-toggle" },
                React.createElement("input", { type: "checkbox", checked: isHierarchical, onChange: (e) => handleHierarchicalChange(e.currentTarget.checked) }),
                React.createElement("span", { className: "input-label-text" },
                    React.createElement(FormattedMessage, { id: "userManagement.workspace.granularPermission.hierarchy" }))),
            React.createElement("div", { className: "gd-granular-permissions__hierarchy-icon" },
                React.createElement(QuestionMarkIcon, { bubbleTextId: workspaceGranularPermissionMessages.hierarchyTooltip.id, width: 14, height: 14 }))),
        React.createElement(DialogListHeader, { title: intl.formatMessage({
                id: "userManagement.workspace.granularPermission.workspaceAccess.title",
            }) }),
        React.createElement("div", { className: "gd-granular-permissions__workspace-access" }, workspacePermissions.map((workspacePermission) => {
            return (React.createElement(WorkspaceAccessPermissionItem, { key: workspacePermission, item: workspacePermission, checked: workspacePermission === selectedWorkspacePermission, onChange: () => handleWorkspacePermissionChange(workspacePermission) }));
        })),
        React.createElement("div", { className: "gd-granular-permissions__workspace-access-description" },
            React.createElement(FormattedMessage, { id: getWorkspaceAccessPermissionDescription(selectedWorkspacePermission).id })),
        React.createElement(DialogListHeader, { title: intl.formatMessage({
                id: "userManagement.workspace.granularPermission.additionalAccess.title",
            }) }),
        React.createElement("div", { className: "gd-granular-permissions__additional-access-wrapper" }, granularItems.map((granularPermission) => {
            return (React.createElement(AdditionalAccessPermissionItem, { key: granularPermission.id, item: granularPermission, checked: selectedGranularPermissions.includes(granularPermission.id), indefinite: isExportPermissionIndefinite(granularPermission.id, selectedGranularPermissions), disabled: isPermissionDisabled(granularPermission.id, selectedWorkspacePermission, selectedGranularPermissions), onChange: () => handleGranularPermissionChange(granularPermission.id) }));
        })),
        showRedundancyWarningMessage ? (React.createElement(Message, { type: "warning", className: "gd-granular-permissions__warning" },
            React.createElement(FormattedMessage, { id: "userManagement.workspace.granularPermission.warning", values: {
                    b: (chunks) => React.createElement("strong", null, chunks),
                } }))) : null));
};
//# sourceMappingURL=GranularPermissions.js.map