// (C) 2019-2023 GoodData Corporation
// eslint-disable-next-line no-restricted-imports -- unfortunately, the get syntax is used heavily here for the supported properties
import get from "lodash/get.js";
import flow from "lodash/flow.js";
import has from "lodash/has.js";
import set from "lodash/set.js";
import keys from "lodash/keys.js";
import isEmpty from "lodash/isEmpty.js";
import cloneDeep from "lodash/cloneDeep.js";
import isNil from "lodash/isNil.js";
import { BucketNames } from "@gooddata/sdk-ui";
import { AXIS } from "../constants/axis.js";
import { getItemsCount, getItemsLocalIdentifiers, getMeasureItems, getAllMeasuresShowOnSecondaryAxis, getStackItems, } from "./bucketHelper.js";
import { PROPERTY_CONTROLS } from "../constants/properties.js";
import { UICONFIG_AXIS } from "../constants/uiConfig.js";
import { OPTIONAL_STACKING_PROPERTIES } from "../constants/supportedProperties.js";
import { bucketsIsEmpty, insightBuckets } from "@gooddata/sdk-model";
export function getSupportedPropertiesControls(visualizationControlsProperties, supportedPropertiesList) {
    const clonedControls = cloneDeep(visualizationControlsProperties);
    if (supportedPropertiesList) {
        return supportedPropertiesList.reduce((props, current) => has(clonedControls, current) ? set(props, current, get(clonedControls, current)) : props, {});
    }
    return {};
}
export function hasColorMapping(properties) {
    return !!properties?.controls?.colorMapping;
}
export function setSecondaryMeasures(referencePoint, axisName) {
    if (!axisName) {
        return referencePoint;
    }
    const newReferencePoint = cloneDeep(referencePoint);
    const path = `${PROPERTY_CONTROLS}.${axisName}`;
    const secondaryAxisProperties = newReferencePoint?.properties?.controls?.[axisName];
    const buckets = newReferencePoint?.buckets ?? [];
    const allMeasures = getMeasureItems(buckets);
    const secondaryMeasures = getAllMeasuresShowOnSecondaryAxis(buckets);
    if (!secondaryAxisProperties && !secondaryMeasures.length) {
        return referencePoint;
    }
    const secondaryAxis = {
        ...secondaryAxisProperties,
        measures: getItemsLocalIdentifiers(secondaryMeasures),
    };
    const axis = {
        0: AXIS.PRIMARY,
        [allMeasures.length]: AXIS.SECONDARY,
    }[secondaryMeasures.length] || AXIS.DUAL;
    set(newReferencePoint, path, secondaryAxis);
    set(newReferencePoint, UICONFIG_AXIS, axis);
    return newReferencePoint;
}
export function isEmptyObject(obj) {
    return obj && keys(obj).length === 0;
}
export function getSupportedProperties(visualizationProperties, supportedPropertiesList) {
    const controls = visualizationProperties?.controls ?? {};
    const supportedControls = getSupportedPropertiesControls(controls, supportedPropertiesList);
    return isEmpty(supportedControls)
        ? {}
        : {
            controls: supportedControls,
        };
}
export function getReferencePointWithSupportedProperties(referencePoint, supportedPropertiesList) {
    const supportedControlsProperties = referencePoint.properties
        ? getSupportedPropertiesControls(referencePoint.properties.controls, supportedPropertiesList)
        : {};
    if (isEmpty(supportedControlsProperties)) {
        const sortItems = referencePoint.properties?.sortItems;
        const sortItemsExpand = sortItems && !isEmpty(sortItems) ? { sortItems } : {};
        return {
            ...referencePoint,
            properties: {
                ...sortItemsExpand,
            },
        };
    }
    const buckets = referencePoint?.buckets ?? [];
    const stackCount = getItemsCount(buckets, BucketNames.STACK);
    const stackMeasuresToPercent = Boolean(supportedControlsProperties.stackMeasuresToPercent);
    if (!stackCount && stackMeasuresToPercent) {
        supportedControlsProperties.stackMeasures = true;
    }
    return {
        ...referencePoint,
        properties: {
            ...referencePoint.properties,
            controls: supportedControlsProperties,
        },
    };
}
export function getReferencePointWithTotalLabelsInitialized(referencePoint) {
    const dataLabelVisibility = referencePoint.properties.controls?.dataLabels?.visible;
    const stacks = getStackItems(referencePoint.buckets);
    // Initialize total labels visibility with data labels visibility value.
    // Initialize if data labels visibility is defined and total labels visibility
    // is not defined and if current chart configuration allows configuring total labels.
    if (!isNil(dataLabelVisibility) &&
        isNil(referencePoint.properties.controls?.dataLabels?.totalsVisible) &&
        !isStackingToPercent(referencePoint.properties) &&
        (stacks.length || isStackingMeasure(referencePoint.properties))) {
        return {
            ...referencePoint,
            properties: {
                ...referencePoint.properties,
                controls: {
                    ...referencePoint.properties.controls,
                    dataLabels: {
                        ...referencePoint.properties.controls.dataLabels,
                        totalsVisible: dataLabelVisibility,
                    },
                },
            },
        };
    }
    return referencePoint;
}
export function isStackingMeasure(properties) {
    return properties?.controls?.stackMeasures ?? false;
}
export function isStackingToPercent(properties) {
    return properties?.controls?.stackMeasuresToPercent ?? false;
}
export function isTotalSectionEnabled(properties) {
    return properties?.controls?.total?.enabled ?? true;
}
export function isDualAxisOrSomeSecondaryAxisMeasure(extReferencePoint, secondaryMeasures) {
    return ((extReferencePoint?.properties?.controls?.dualAxis ?? true) ||
        secondaryMeasures.some((item) => item?.showOnSecondaryAxis));
}
export function removeImmutableOptionalStackingProperties(referencePoint, supportedPropertiesList) {
    const buckets = referencePoint?.buckets ?? [];
    let immutableProperties = [];
    if (getItemsCount(buckets, BucketNames.MEASURES) <= 1) {
        immutableProperties = OPTIONAL_STACKING_PROPERTIES;
    }
    if (getItemsCount(buckets, BucketNames.STACK)) {
        immutableProperties = OPTIONAL_STACKING_PROPERTIES.slice(0, 1);
    }
    return supportedPropertiesList.filter((property) => !immutableProperties.some((immutableProperty) => immutableProperty === property));
}
// mapping between AD and SDK values
const AXIS_NAME_POSITION_MAPPING = {
    auto: "middle",
    bottom: "low",
    middle: "middle",
    top: "high",
    left: "low",
    center: "middle",
    right: "high",
};
const AXIS_TYPES = ["xaxis", "yaxis", "secondary_xaxis", "secondary_yaxis"];
export function getHighchartsAxisNameConfiguration(controlProperties, enableAxisNameConfiguration = false) {
    const axisProperties = AXIS_TYPES.reduce((result, axis) => {
        const axisNameConfig = controlProperties?.[axis]?.name;
        if (isEmpty(axisNameConfig)) {
            return result;
        }
        axisNameConfig.position =
            AXIS_NAME_POSITION_MAPPING[enableAxisNameConfiguration ? axisNameConfig.position : AXIS_NAME_POSITION_MAPPING.auto];
        result[axis] = {
            ...controlProperties[axis],
            name: axisNameConfig,
        };
        return result;
    }, {});
    return {
        ...controlProperties,
        ...axisProperties,
    };
}
export function getDataPointsConfiguration(controlProperties, enableHidingOfDataPoints = false) {
    if (enableHidingOfDataPoints) {
        const dataPointsVisible = controlProperties.dataPoints?.visible;
        return {
            ...controlProperties,
            dataPoints: {
                visible: dataPointsVisible !== undefined ? dataPointsVisible : "auto",
            },
        };
    }
    return {
        ...controlProperties,
        dataPoints: {
            visible: undefined,
        },
    };
}
export function getColumnWidthsFromProperties(visualizationProperties) {
    return visualizationProperties?.controls?.columnWidths;
}
export function getMeasureGroupDimensionFromProperties(visualizationProperties) {
    return visualizationProperties?.controls?.measureGroupDimension;
}
export function getColumnHeadersPositionFromProperties(visualizationProperties) {
    return visualizationProperties?.controls?.columnHeadersPosition;
}
export function getLegendConfiguration(controlProperties, insight) {
    const legendPosition = getLegendPosition(controlProperties, insight);
    set(controlProperties, "legend.position", legendPosition);
    return controlProperties;
}
export function getLegendConfigurationDashboardsEnv(controlProperties, options, enableKDWidgetCustomHeight) {
    const legendPosition = getLegendPositionDashboardsEnv(controlProperties, options);
    set(controlProperties, "legend.position", legendPosition);
    const legendResponsiveness = enableKDWidgetCustomHeight ? "autoPositionWithPopup" : true;
    set(controlProperties, "legend.responsive", legendResponsiveness);
    return controlProperties;
}
export function getChartSupportedControls(controlProperties, insight, settings) {
    return flow((c) => cloneDeep(c ?? {}), (c) => getLegendConfiguration(c, insight), (c) => getHighchartsAxisNameConfiguration(c, settings?.enableAxisNameConfiguration), (c) => getDataPointsConfiguration(c, settings?.enableHidingOfDataPoints))(controlProperties);
}
export function getChartSupportedControlsDashboardsEnv(controlProperties, options, settings) {
    return flow((c) => cloneDeep(c ?? {}), (c) => getLegendConfigurationDashboardsEnv(c, options, settings?.enableKDWidgetCustomHeight), (c) => getHighchartsAxisNameConfiguration(c, settings?.enableAxisNameConfiguration), (c) => getDataPointsConfiguration(c, settings?.enableHidingOfDataPoints))(controlProperties);
}
function getLegendPosition(controlProperties, insight) {
    const legendPosition = controlProperties?.legend?.position ?? "auto";
    return legendPosition === "auto" && isStacked(insight) ? "right" : legendPosition;
}
function getLegendPositionDashboardsEnv(controlProperties, options) {
    const legendPosition = controlProperties?.legend?.position ?? "auto";
    const width = options.dimensions?.width;
    return width !== undefined && width <= getMaxWidthForCollapsedLegend(legendPosition)
        ? "top"
        : legendPosition;
}
function isStacked(insight) {
    return !bucketsIsEmpty(insightBuckets(insight, BucketNames.STACK, BucketNames.SEGMENT));
}
const MAX_WIDTH_FOR_COLLAPSED_LEGEND = 440;
const MAX_WIDTH_FOR_COLLAPSED_AUTO_LEGEND = 610;
function getMaxWidthForCollapsedLegend(legendPosition) {
    return legendPosition === "auto" ? MAX_WIDTH_FOR_COLLAPSED_AUTO_LEGEND : MAX_WIDTH_FOR_COLLAPSED_LEGEND;
}
export function getPivotTableProperties(settings, properties) {
    const { enablePivotTableTransposition, enableColumnHeadersPosition } = settings;
    return {
        ...(enablePivotTableTransposition === true
            ? {
                measureGroupDimension: getMeasureGroupDimensionFromProperties(properties),
            }
            : {}),
        ...(enableColumnHeadersPosition === true
            ? {
                columnHeadersPosition: getColumnHeadersPositionFromProperties(properties),
            }
            : {}),
    };
}
//# sourceMappingURL=propertiesHelper.js.map