import { Bubble, BubbleHoverTrigger, getDateTimeConfig, isActionKey, UiButton, UiIcon, UiIconButton, useToastMessage, } from "@gooddata/sdk-ui-kit";
import compact from "lodash/compact.js";
import React, { useCallback } from "react";
import { defineMessages, FormattedDate, FormattedMessage, FormattedTime, useIntl } from "react-intl";
import { bem } from "../bem.js";
import { Popup } from "../components/Popup.js";
import { Tooltip } from "../components/Tooltip.js";
import { NotificationFiltersDetail } from "../NotificationFiltersDetail/NotificationFiltersDetail.js";
import { NotificationTriggerDetail } from "../NotificationTriggersDetail/NotificationTriggersDetail.js";
import { downloadFiles } from "../utils/downloadFiles.js";
const { b, e } = bem("gd-ui-ext-notification");
/**
 * @internal
 */
export function Notification({ notification, markNotificationAsRead, onNotificationClick, closeNotificationsPanel, }) {
    const intl = useIntl();
    const { addWarning } = useToastMessage();
    const notificationTitle = getNotificationTitle(notification);
    const filters = notification.details.data.filters;
    const hasFilters = !!filters?.length;
    const isAlertNotification = notification.notificationType === "alertNotification";
    const isScheduleNotification = notification.notificationType === "scheduleNotification";
    const isSliced = isAlertNotification && !!notification.details.data.alert.attribute;
    const hasTriggers = isAlertNotification && !!notification.details.data.alert.totalValueCount;
    const exports = compact([
        notification.details.data.visualExports,
        notification.details.data.tabularExports,
        notification.details.data.imageExports,
        notification.details.data.rawExports,
        notification.details.data.slidesExports,
        notification.details.data.dashboardTabularExports,
    ]).flatMap((arr) => arr ?? []);
    const hasExports = exports.length > 0;
    const fileExpiresAt = exports?.[0]?.expiresAt ?? null;
    const isExpired = fileExpiresAt != null && new Date(fileExpiresAt) < new Date();
    const { isError, errorTitle, errorMessage, traceId } = getNotificationErrorInfo(notification, exports);
    const actions = compact([
        hasFilters && isAlertNotification && React.createElement(NotificationFiltersDetail, { filters: filters }),
        hasTriggers && isSliced && React.createElement(NotificationTriggerDetail, { notification: notification }),
        hasExports && React.createElement(FileLink, { notification: notification }),
        hasExports && fileExpiresAt && React.createElement(FileExpiration, { fileExpiresAt: fileExpiresAt, isExpired: isExpired }),
    ]);
    const handleNotificationClick = useCallback(() => {
        if (!notification.isRead) {
            markNotificationAsRead(notification.id);
        }
        if (notification.notificationType === "scheduleNotification") {
            if (isExpired) {
                addWarning(messages.notificationExpired);
                return;
            }
            const downloadableExports = compact(exports.map(mapToDownloadableFile));
            if (downloadableExports.length > 0) {
                downloadFiles(downloadableExports);
            }
        }
        else {
            closeNotificationsPanel();
            onNotificationClick?.(notification);
        }
    }, [
        onNotificationClick,
        notification,
        isExpired,
        markNotificationAsRead,
        closeNotificationsPanel,
        addWarning,
        exports,
    ]);
    const handleKeyDown = (event) => {
        if (isActionKey(event)) {
            event.preventDefault();
            handleNotificationClick();
        }
    };
    return (React.createElement(BubbleHoverTrigger, { enabled: hasExports && !isExpired && !isError },
        React.createElement("div", { className: b({ isRead: notification.isRead }), onClick: handleNotificationClick, onKeyDown: handleKeyDown, role: "listitem", tabIndex: 0, "aria-label": intl.formatMessage(messages.alertNotificationTypeLabel) },
            React.createElement("div", { className: e("icon") },
                !notification.isRead && React.createElement("div", { className: e("unread-status") }),
                isAlertNotification ? (React.createElement(UiIcon, { type: "alert", size: 14, color: "complementary-6", ariaHidden: true })) : (React.createElement(UiIcon, { type: "download", size: 14, color: "complementary-6", ariaHidden: true }))),
            React.createElement("div", { className: e("details") },
                React.createElement("div", { className: e("title", { isRead: notification.isRead }), title: notificationTitle },
                    isScheduleNotification ? (React.createElement(FormattedMessage, { id: "notifications.panel.download" })) : null,
                    " ",
                    notificationTitle),
                isError ? (React.createElement("div", { className: e("error") },
                    React.createElement("div", { className: e("error-icon") },
                        React.createElement(UiIcon, { type: "crossCircle", size: 12, color: "error" })),
                    React.createElement("div", null,
                        errorTitle,
                        " ",
                        React.createElement(Popup, { popup: React.createElement("div", { className: e("error-popup") },
                                errorMessage,
                                React.createElement("br", null),
                                React.createElement(FormattedMessage, { id: "notifications.panel.error.traceId" }),
                                ":",
                                " ",
                                traceId) }, ({ toggle, id }) => (React.createElement(UiButton, { dataId: "notification-error", id: id, onClick: (e) => {
                                e.stopPropagation();
                                toggle();
                            }, onKeyDown: (e) => {
                                e.stopPropagation();
                            }, variant: "tertiary", size: "small", label: intl.formatMessage({
                                id: "notifications.panel.error.learnMore",
                            }) })))))) : actions.length ? (React.createElement("div", { className: e("links") }, actions.map((action, index) => (React.createElement(React.Fragment, { key: index },
                    !!index && "・",
                    action))))) : null),
            React.createElement("div", { className: e("time") },
                React.createElement(NotificationTime, { config: getDateTimeConfig(notification.createdAt) })),
            React.createElement("div", { className: e("mark-as-read-button") },
                React.createElement(Tooltip, { tooltip: intl.formatMessage(messages.markAsRead) },
                    React.createElement(UiIconButton, { onClick: (e) => {
                            e.stopPropagation();
                            markNotificationAsRead(notification.id);
                        }, onKeyDown: (e) => {
                            e.stopPropagation();
                        }, icon: "check", size: "small", variant: "tertiary", label: intl.formatMessage(messages.markAsRead) })))),
        React.createElement(Bubble, { alignPoints: [{ align: "bc tc" }] },
            React.createElement(FormattedMessage, { id: "notifications.panel.downloadFilesHint" }))));
}
function mapToDownloadableFile(exportFile) {
    if (!exportFile.fileUri) {
        return undefined;
    }
    return {
        url: exportFile.fileUri,
        fileName: exportFile.fileName,
    };
}
function getNotificationTitle(notification) {
    if (notification.notificationType === "alertNotification") {
        return getAlertNotificationTitle(notification.details.data.alert);
    }
    else if (notification.notificationType === "scheduleNotification") {
        return notification.details.data.automation.title;
    }
    return notification.details.message;
}
function getAlertNotificationTitle(alertDescription) {
    const metric = alertDescription.metric;
    const condition = alertDescription.condition;
    const formattedThreshold = alertDescription.formattedThreshold;
    // TODO: translate (backend is currently returning English, should return translation strings)
    return `${metric} is ${condition} ${formattedThreshold}`;
}
function getNotificationErrorInfo(notification, exports) {
    if (notification.notificationType === "alertNotification" &&
        notification.details.data.alert?.status !== "SUCCESS") {
        return {
            isError: true,
            errorTitle: React.createElement(FormattedMessage, { id: "notifications.panel.error.alert.title" }),
            errorMessage: notification.details.data.alert.errorMessage ?? "",
            traceId: notification.details.data.alert.traceId,
        };
    }
    if (exports.some((exportResult) => exportResult.status !== "SUCCESS")) {
        return {
            isError: true,
            errorTitle: React.createElement(FormattedMessage, { id: "notifications.panel.error.schedule.title" }),
            errorMessage: exports.find((exportResult) => exportResult.status !== "SUCCESS")?.errorMessage ?? "",
            traceId: exports.find((exportResult) => exportResult.status !== "SUCCESS")?.traceId,
        };
    }
    if (notification.details.webhookMessageType === "automation-task.limit-exceeded") {
        return {
            isError: true,
            errorTitle: React.createElement(FormattedMessage, { id: "notifications.panel.error.schedule.title" }),
            errorMessage: React.createElement(FormattedMessage, { id: "notifications.panel.error.limitExceeded.message" }),
            traceId: undefined,
        };
    }
    return { isError: false, errorTitle: undefined, errorMessage: undefined, traceId: undefined };
}
const NotificationTime = ({ config }) => {
    if (config.isToday) {
        return React.createElement(FormattedTime, { value: config.date, format: "hhmm", hour12: false });
    }
    else if (config.isYesterday) {
        return (React.createElement("span", null,
            React.createElement(FormattedMessage, { id: "gs.date.yesterday" })));
    }
    return React.createElement(FormattedDate, { value: config.date, format: "shortWithYear" });
};
const FileLink = ({ notification }) => {
    return (React.createElement(BubbleHoverTrigger, { eventsOnBubble: true },
        React.createElement("a", { href: notification.details.data.automation.dashboardURL, onClick: (e) => e.stopPropagation(), onKeyDown: (e) => e.stopPropagation(), className: e("link") },
            React.createElement(FormattedMessage, { id: "notifications.panel.dashboardLink", values: { dashboardTitle: notification.details.data.automation.dashboardTitle } })),
        React.createElement(Bubble, { alignPoints: [{ align: "bc tc" }] },
            " ",
            React.createElement(FormattedMessage, { id: "notifications.panel.dashboardLinkHint" }),
            " ")));
};
const FileExpiration = ({ fileExpiresAt, isExpired }) => {
    if (isExpired) {
        return (React.createElement("span", null,
            React.createElement(FormattedMessage, { id: "notifications.panel.linkHasExpired" })));
    }
    return (React.createElement(BubbleHoverTrigger, { eventsOnBubble: true },
        React.createElement("span", null,
            React.createElement(FormattedMessage, { id: "notifications.panel.expiresOn", values: {
                    date: (React.createElement(FormattedDate, { value: getDateTimeConfig(fileExpiresAt).date, format: "shortWithoutYear" })),
                } })),
        React.createElement(Bubble, { alignPoints: [{ align: "bc tc" }] },
            " ",
            React.createElement(FormattedTime, { value: getDateTimeConfig(fileExpiresAt).date, format: "hhmm" }),
            " ")));
};
const messages = defineMessages({
    markAsRead: {
        id: "notifications.panel.markAsRead",
    },
    alertNotificationTypeLabel: {
        id: "notifications.panel.alert.notification.type.label",
    },
    notificationExpired: {
        id: "notifications.panel.toast.warning.filesExpired",
        values: {
            strong: (text) => React.createElement("strong", null, text),
        },
    },
});
//# sourceMappingURL=Notification.js.map